@@ -15,6 +15,7 @@
 
 import logging
 import re
+import time
 
 from ironicclient import exc as ironicexp
 import six
@@ -231,15 +232,18 @@ def register_ironic_node(service_host, node, client=None, blocking=None):
                   if node.get(v) is not None}
 
     if 'capabilities' in node:
-        properties.update({"capabilities":
-                          six.text_type(node.get('capabilities'))})
+        caps = node['capabilities']
+        if isinstance(caps, dict):
+            caps = dict_to_capabilities(caps)
+        properties.update({"capabilities": six.text_type(caps)})
 
     create_map = {"driver": node["pm_type"],
                   "properties": properties,
                   "driver_info": driver_info}
 
-    if 'name' in node:
-        create_map.update({"name": six.text_type(node.get('name'))})
+    for field in ('name', 'uuid'):
+        if field in node:
+            create_map.update({field: six.text_type(node[field])})
 
     node_id = handler.unique_id_from_fields(node)
     LOG.debug('Registering node %s with ironic.', node_id)
@@ -265,7 +269,7 @@ def register_ironic_node(service_host, node, client=None, blocking=None):
 
 def _populate_node_mapping(client):
     LOG.debug('Populating list of registered nodes.')
-    node_map = {'mac': {}, 'pm_addr': {}}
+    node_map = {'mac': {}, 'pm_addr': {}, 'uuids': set()}
     nodes = client.node.list(detail=True)
     for node in nodes:
         for port in client.node.list_ports(node.uuid):
@@ -276,6 +280,8 @@ def _populate_node_mapping(client):
         if unique_id:
             node_map['pm_addr'][unique_id] = node.uuid
 
+        node_map['uuids'].add(node.uuid)
+
     return node_map
 
 
@@ -294,6 +300,10 @@ def _get_node_id(node, handler, node_map):
         except KeyError:
             pass
 
+    uuid = node.get('uuid')
+    if uuid and uuid in node_map['uuids']:
+        candidates.add(uuid)
+
     if len(candidates) > 1:
         raise exception.InvalidNode('Several candidates found for the same '
                                     'node data: %s' % candidates,
@@ -316,6 +326,8 @@ def _update_or_register_ironic_node(service_host, node, node_map, client=None):
                             ('disk', '/properties/local_gb'),
                             ('arch', '/properties/cpu_arch'),
                             ('name', '/name'),
+                            ('kernel_id', '/driver_info/deploy_kernel'),
+                            ('ramdisk_id', '/driver_info/deploy_ramdisk'),
                             ('capabilities', '/properties/capabilities')]:
             if field in node:
                 patched[path] = node.pop(field)
@@ -326,6 +338,8 @@ def _update_or_register_ironic_node(service_host, node, node_map, client=None):
 
         node_patch = []
         for key, value in patched.items():
+            if key == 'uuid':
+                continue  # not needed during update
             node_patch.append({'path': key,
                                'value': six.text_type(value),
                                'op': 'add'})
@@ -348,9 +362,134 @@ def _clean_up_extra_nodes(seen, client, remove=False):
             LOG.debug('Extra registered node %s found.' % node)
 
 
+def wait_for_provision_state(baremetal_client, node_uuid, provision_state,
+                             loops=10, sleep=1):
+    """Wait for a given Provisioning state in Ironic
+
+    Updating the provisioning state is an async operation, we
+    need to wait for it to be completed.
+
+    :param baremetal_client: Instance of Ironic client
+    :type  baremetal_client: ironicclient.v1.client.Client
+
+    :param node_uuid: The Ironic node UUID
+    :type  node_uuid: str
+
+    :param provision_state: The provisioning state name to wait for
+    :type  provision_state: str
+
+    :param loops: How many times to loop
+    :type loops: int
+
+    :param sleep: How long to sleep between loops
+    :type sleep: int
+
+    :raises exceptions.StateTransitionFailed: if node.last_error is set
+    """
+
+    for _l in range(0, loops):
+
+        # This will throw an exception if the UUID is not found, so no need to
+        # check for node == None
+        node = baremetal_client.node.get(node_uuid)
+
+        if node.provision_state == provision_state:
+            LOG.info('Node %s set to provision state %s',
+                     node_uuid, provision_state)
+            return
+
+        # node.last_error should be None after any successful operation
+        if node.last_error:
+            raise exception.StateTransitionFailed(node, provision_state)
+
+        time.sleep(sleep)
+
+    raise exception.Timeout(
+        "Node %(uuid)s did not reach provision state %(state)s. "
+        "Now in state %(actual)s." % {
+            'uuid': node_uuid,
+            'state': provision_state,
+            'actual': node.provision_state
+        }
+    )
+
+
+def set_nodes_state(baremetal_client, nodes, transition, target_state,
+                    skipped_states=()):
+    """Make all nodes available in the baremetal service for a deployment
+
+    For each node whose provision_state is not in skipped_states, apply the
+    specified transition and wait until its provision_state is target_state.
+
+    :param baremetal_client: Instance of Ironic client
+    :type  baremetal_client: ironicclient.v1.client.Client
+
+    :param nodes: List of Baremetal Nodes
+    :type  nodes: [ironicclient.v1.node.Node]
+
+    :param transition: The state to set for a node. The full list of states
+                       can be found in ironic.common.states.
+    :type  transition: string
+
+    :param target_state: The expected result state for a node. For example when
+                         transitioning to 'manage' the result is 'manageable'
+    :type  target_state: string
+
+    :param skipped_states: A set of states to skip, for example 'active' nodes
+                           are already deployed and the state can't always be
+                           changed.
+    :type  skipped_states: iterable of strings
+
+    :raises exception.Timeout: if a node takes too long to reach target state
+
+    :return List of nodes whose provision states have been altered. These
+            objects will be stale, and will not reflect the real node's current
+            provision_state.
+    """
+
+    log = logging.getLogger(__name__ + ".set_nodes_state")
+    altered_nodes = []
+
+    for node in nodes:
+
+        if node.provision_state in skipped_states:
+            continue
+
+        log.debug(
+            "Setting provision state from '{0}' to '{1}' for Node {2}"
+            .format(node.provision_state, transition, node.uuid))
+
+        baremetal_client.node.set_provision_state(node.uuid, transition)
+        try:
+            wait_for_provision_state(baremetal_client, node.uuid, target_state)
+        except exception.StateTransitionFailed as e:
+            log.error("FAIL: {0}".format(e))
+        except exception.Timeout as e:
+            log.error("FAIL: {0}".format(e))
+        altered_nodes.append(node)
+
+    return altered_nodes
+
+
 def register_all_nodes(service_host, nodes_list, client=None, remove=False,
                        blocking=True, keystone_client=None, glance_client=None,
-                       kernel_name=None, ramdisk_name=None):
+                       kernel_name=None, ramdisk_name=None,
+                       provide=True):
+    """Register all nodes in nodes_list in the baremetal service.
+
+    :param service_host: The host providing the baremetal API.
+    :param nodes_list: The list of nodes to register.
+    :param client: An Ironic client object.
+    :param remove: Should nodes not in the list be removed?
+    :param blocking: Ignored.
+    :param keystone_client: Ignored.
+    :param glance_client: A Glance client object, for fetching ramdisk images.
+    :param kernel_name: Glance ID of the kernel to use for the nodes.
+    :param ramdisk_name: Glance ID of the ramdisk to use for the nodes.
+    :param provide: Should the node be transitioned to AVAILABLE state?
+    :return: list of node objects representing the new nodes.
+    """
+
     LOG.debug('Registering all nodes.')
     node_map = _populate_node_mapping(client)
 
@@ -372,4 +511,43 @@ def register_all_nodes(service_host, nodes_list, client=None, remove=False,
         seen.append(node)
 
     _clean_up_extra_nodes(seen, client, remove=remove)
+
+    if provide:
+        manageable_nodes = set_nodes_state(
+            client, seen, "manage", "manageable",
+            skipped_states={'manageable', 'available'}
+        )
+        set_nodes_state(
+            client, manageable_nodes, "provide", "available",
+            skipped_states={'available'}
+        )
+
     return seen
+
+
+def register_all_nodes_compat(service_host, *args, **kwargs):
+    """This is a compatibility shim for the CLI
+
+    This is to help remove the unused service_host arg. It will be removed in
+    the following commit once python-tripleoclient has migrated to the updated
+    register_all_nodes.
+    """
+
+    # TODO(d0ugal): Remove this shim once python-tripleoclient has been updated
+    register_all_nodes(service_host, *args, **kwargs)
+
+
+def dict_to_capabilities(caps_dict):
+    """Convert a dictionary into a string with the capabilities syntax."""
+    return ','.join(["%s:%s" % (key, value)
+                     for key, value in caps_dict.items()
+                     if value is not None])
+
+
+def capabilities_to_dict(caps):
+    """Convert the Node's capabilities into a dictionary."""
+    if not caps:
+        return {}
+    if isinstance(caps, dict):
+        return caps
+    return dict([key.split(':', 1) for key in caps.split(',')])
